package org.springblade.pson.controller;


import java.util.Date;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.EventPropMap;
import org.springblade.pson.service.IEventPropMapService;
import org.springblade.pson.service.IPropService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;

/**
 * <p>
 * 事件属性关联表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-06-07
 */
@RestController
@RequestMapping("/eventPropMap")
@AllArgsConstructor
@Api(value = "事件属性关联表", tags = "事件属性关联表管理接口")
public class EventPropMapController extends BladeController {

	private final IEventPropMapService eventPropMapService;

	private final IPropService propService;

	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增事件属性")
	public R save(@Valid @RequestBody EventPropMap eventPropMap) {
		eventPropMap.setCreateTime(new Date());
		return R.status(eventPropMapService.save(eventPropMap));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除事件属性")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(eventPropMapService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改事件属性")
	public R update(@Valid @RequestBody EventPropMap eventPropMap) {
		return R.status(eventPropMapService.updateById(eventPropMap));
	}


	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "分页")
	public R<IPage<EventPropMap>> list(EventPropMap eventPropMap, Query query) {
		IPage<EventPropMap> pages = eventPropMapService.page(Condition.getPage(query), Condition.getQueryWrapper(eventPropMap));
		return R.data(pages);
	}

	@GetMapping("/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "列表")
	public R list(Long eventId) {
//		List<EventPropMap> list = eventPropMapService.list(
//			new LambdaQueryWrapper<EventPropMap>()
//				.isNotNull(EventPropMap::getId)
//				.eq(EventPropMap::getEventId,eventId)
//		);
//
//		List<EventPropMapVO> res = list.stream()
//			.map(obj ->{
//				EventPropMapVO vo = new EventPropMapVO();
//				BeanUtils.copyProperties(obj,vo);
//				Prop byId = propService.getById(obj.getPropId());
//				vo.setPropName(byId.getName());
//				return vo;
//			}).collect(Collectors.toList());
//		return R.data(res);
		return R.data(eventPropMapService.getList(eventId));
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情")
	public R<EventPropMap> detail(EventPropMap eventPropMap) {
		EventPropMap detail = eventPropMapService.getOne(Condition.getQueryWrapper(eventPropMap));
		return R.data(detail);
	}

}

