package org.springblade.pson.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.Event;
import org.springblade.pson.entity.ProjectEventMap;
import org.springblade.pson.service.IEventService;
import org.springblade.pson.service.IProjectEventMapService;
import org.springblade.pson.vo.ProjectEventMapVO;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 项目事件关联表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-06-07
 */
@RestController
@RequestMapping("/projectEventMap")
@AllArgsConstructor
@Api(value = "项目事件关联表", tags = "项目事件关联表管理接口")
public class ProjectEventMapController {

	private final IProjectEventMapService projectEventMapService;

	private final IEventService eventService;

	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增项目事件")
	public R save(@Valid @RequestBody ProjectEventMap projectEventMap) {
		if(projectEventMap.getLevel() == null){
			projectEventMap.setLevel(0);
		}
		return R.status(projectEventMapService.save(projectEventMap));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除项目事件")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(projectEventMapService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改项目事件")
	public R update(@Valid @RequestBody ProjectEventMap projectEventMap) {
		return R.status(projectEventMapService.updateById(projectEventMap));
	}

	@PostMapping("/updateSorts")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改项目事件排序")
	public R updateSorts(@RequestBody List<ProjectEventMap> mapList) {
		return R.status(projectEventMapService.updateBatchById(mapList));
	}


	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "分页")
	public R<IPage<ProjectEventMap>> list(ProjectEventMap projectEventMap, Query query) {
		IPage<ProjectEventMap> pages = projectEventMapService.page(Condition.getPage(query), Condition.getQueryWrapper(projectEventMap));
		return R.data(pages);
	}

	@GetMapping("/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "列表")
	public R list(Integer projectId) {
		List<ProjectEventMap> list = projectEventMapService.list(
			new LambdaQueryWrapper<ProjectEventMap>()
				.isNotNull(ProjectEventMap::getId)
				.eq(ProjectEventMap::getProjectId,projectId)
		);

		List<ProjectEventMapVO> res = list.stream()
			.map(obj ->{
				ProjectEventMapVO vo = new ProjectEventMapVO();
				vo.setId(obj.getId());
				vo.setEventId(obj.getEventId());
				vo.setProjectId(obj.getProjectId());
				vo.setLevel(obj.getLevel());
				vo.setSort(obj.getSort());
				vo.setPropIdJson(obj.getPropIdJson());
				Event byId = eventService.getById(obj.getEventId());
				vo.setEventName(byId.getName());
				return vo;
			}).collect(Collectors.toList());
		return R.data(res);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情")
	public R<ProjectEventMap> detail(ProjectEventMap projectEventMap) {
		ProjectEventMap detail = projectEventMapService.getOne(Condition.getQueryWrapper(projectEventMap));
		return R.data(detail);
	}


}

