package org.springblade.pson.controller;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.Prop;
import org.springblade.pson.service.IPropService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;

/**
 * <p>
 * 属性表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-06-07
 */
@RestController
@RequestMapping("/prop")
@AllArgsConstructor
@Api(value = "属性", tags = "属性管理接口")
public class PropController extends BladeController {

	private final IPropService propService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增属性")
	public R save(@Valid @RequestBody Prop prop) {
		prop.setCreateTime(new Date());
		return R.status(propService.save(prop));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除属性")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(propService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改属性")
	public R update(@Valid @RequestBody Prop prop) {
		return R.status(propService.updateById(prop));
	}


	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "分页")
	public R<IPage<Prop>> list(Prop prop, Query query,String propName) {
		IPage<Prop> pages = propService.page(Condition.getPage(query), Condition.getQueryWrapper(prop));

		if(StrUtil.isNotEmpty(propName)){
			pages = propService.page(Condition.getPage(query), new QueryWrapper<Prop>().like("name",propName));
		}
		return R.data(pages);
	}

	@GetMapping("/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "列表")
	public R list(Integer type) {
		LambdaQueryWrapper<Prop> propLambdaQueryWrapper = new LambdaQueryWrapper<>();
		propLambdaQueryWrapper.isNotNull(Prop::getId);
		if(type != null){
			propLambdaQueryWrapper.eq(Prop::getType,type);
		}
		List<Prop> list = propService.list(propLambdaQueryWrapper);
		return R.data(list);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情")
	public R<Prop> detail(Prop prop) {
		Prop detail = propService.getOne(Condition.getQueryWrapper(prop));
		return R.data(detail);
	}


	@GetMapping("/select/list")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "事件下拉列表")
	public R selectList() {
		List<Prop> list = propService.list(
			new LambdaQueryWrapper<Prop>()
				.isNotNull(Prop::getId)
		);
		List<Map<String, Object>> data = list.stream()
			.map(obj -> {
				Map<String, Object> map = new HashMap<>();
				map.put("id", obj.getId());
				map.put("name", obj.getName());
				return map;
			}).collect(Collectors.toList());
		return R.data(data);
	}
}

