/**
 * 
 */
package org.springblade.useranalysis.controller;

import java.util.Date;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.useranalysis.entity.AdsUserprofileAnalysisBook;
import org.springblade.useranalysis.service.normal.AdsUserprofileAnalysisBookService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 *	用户画像-行为分析-工作薄 操作处理
 *
 * @author wookvn
 * @date   2021-08-19
 */
@RestController
@RequestMapping("/userprofileAnalysisBook")
@Api(value = "用户画像-行为分析-工作薄相关接口", tags = {"用户画像-行为分析-工作薄相关接口"})
public class AdsUserprofileAnalysisBookController extends BladeController {

	@Autowired
	private AdsUserprofileAnalysisBookService adsUserprofileAnalysisBookService;
	
	/**
	 * 	列表数据
	 * @param adsUserprofileAnalysisBook
	 * @return
	 */
	@PostMapping("/list")
	@ApiOperation(value = "分页查询用户画像-行为分析-工作薄信息", httpMethod = "POST")
	public R<IPage<AdsUserprofileAnalysisBook>> list(AdsUserprofileAnalysisBook adsUserprofileAnalysisBook, Query query){
		LambdaQueryWrapper<AdsUserprofileAnalysisBook> queryWrapper = Wrappers.<AdsUserprofileAnalysisBook>lambdaQuery();
		if(adsUserprofileAnalysisBook.getEventType() != null) {
			queryWrapper.eq(AdsUserprofileAnalysisBook::getEventType, adsUserprofileAnalysisBook.getEventType());
		}
		//查询共享的
		if(adsUserprofileAnalysisBook.getShareStatus() != null && adsUserprofileAnalysisBook.getShareStatus() == 1) {
			queryWrapper.and(wrapper -> {
				wrapper.and(tmpWrapper -> {
					if(adsUserprofileAnalysisBook.getDefaultStatus() != null) {
						tmpWrapper.eq(AdsUserprofileAnalysisBook::getDefaultStatus, adsUserprofileAnalysisBook.getDefaultStatus());
					}
					tmpWrapper.eq(AdsUserprofileAnalysisBook::getCreateUserId, SecureUtil.getUserId());
				});
				wrapper.or(tmpWrapper -> {
					tmpWrapper.eq(AdsUserprofileAnalysisBook::getShareStatus, adsUserprofileAnalysisBook.getShareStatus());
				});
			});
		}else {
			if(adsUserprofileAnalysisBook.getDefaultStatus() != null) {
				queryWrapper.eq(AdsUserprofileAnalysisBook::getDefaultStatus, adsUserprofileAnalysisBook.getDefaultStatus());
			}
			queryWrapper.eq(AdsUserprofileAnalysisBook::getCreateUserId, SecureUtil.getUserId());
		}
		queryWrapper.orderByDesc(AdsUserprofileAnalysisBook::getId);
		return R.data(adsUserprofileAnalysisBookService.page(Condition.getPage(query), queryWrapper));
	}
	
	/**
	 * 单条数据查询接口
	 * @param oprCenterId
	 * @return
	 */
	@PostMapping("/info")
	@ApiOperation(value = "根据ID获取用户画像-行为分析-工作薄信息", httpMethod = "POST")
	public R<AdsUserprofileAnalysisBook> info(Integer id) {
		return R.data(adsUserprofileAnalysisBookService.getById(id));
	}
	
	/**
	 * 	添加数据接口
	 * @param adsUserprofileAnalysisBook
	 * @return
	 */
	@PostMapping("/add")
	@ApiOperation(value = "添加用户画像-行为分析-工作薄", httpMethod = "POST")
	public R<String> doAdd(@RequestBody AdsUserprofileAnalysisBook adsUserprofileAnalysisBook) {
		adsUserprofileAnalysisBook.setCreateBy(SecureUtil.getUserName());
		adsUserprofileAnalysisBook.setCreateUserId(SecureUtil.getUserId());
		adsUserprofileAnalysisBook.setCreateTime(new Date());
		return R.status(adsUserprofileAnalysisBookService.save(adsUserprofileAnalysisBook));
	}
	
	/**
	 *	修改数据接口
	 * @param adsUserprofileAnalysisBook
	 * @return
	 */
	@PostMapping("/update")
	@ApiOperation(value = "根据ID更新用户画像-行为分析-工作薄", httpMethod = "POST")
	public R<String> doUpdate(@RequestBody AdsUserprofileAnalysisBook adsUserprofileAnalysisBook) {
		if(adsUserprofileAnalysisBook == null || adsUserprofileAnalysisBook.getId() == null || adsUserprofileAnalysisBook.getId() <= 0) {
			return R.fail("参数错误，修改失败。");
		}
		adsUserprofileAnalysisBook.setUpdateBy(SecureUtil.getUserName());
		adsUserprofileAnalysisBook.setUpdateTime(new Date());
		return R.status(adsUserprofileAnalysisBookService.updateById(adsUserprofileAnalysisBook));
	}
	
	/**
	 * 	删除数据接口
	 * @param id
	 * @return
	 */
	@PostMapping("/delete")
	@ApiOperation(value = "根据ID删除用户画像-行为分析-工作薄", httpMethod = "POST")
	public R<String> doDelete(@RequestParam Integer id) {
		return R.status(adsUserprofileAnalysisBookService.removeById(id));
	}
	
}