package org.springblade.useranalysis.controller;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.pson.entity.Project;
import org.springblade.pson.entity.ProjectEventMap;
import org.springblade.pson.entity.Prop;
import org.springblade.pson.service.IEventService;
import org.springblade.pson.service.IProjectEventMapService;
import org.springblade.pson.service.IProjectService;
import org.springblade.pson.service.IPropService;
import org.springblade.pson.vo.EventVO;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.common.utils.ErrorEnum;
import org.springblade.useranalysis.dto.RequestDistributeAnalysis;
import org.springblade.useranalysis.dto.RequestEventAnalysis;
import org.springblade.useranalysis.dto.RequestHopperAnalysis;
import org.springblade.useranalysis.dto.RequestRetentionAnalysis;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.service.UserEventService;
import org.springblade.useranalysis.service.normal.DynamicQueryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 分析模型列表页
 */
@RestController
@RequestMapping("/v1/modelList")
@Api(value = "clickhouse", tags = "事件分析模型列表页")
public class AnalysisModelListController {

	@Autowired
	private UserEventService userEventService;

	@Autowired
	private IProjectService projectService;

	@Autowired
	private IEventService eventService;
	
	@Autowired
	private IPropService propService;

	@Autowired
	private IProjectEventMapService projectEventMapService;
	
	@Autowired
	private DynamicQueryService dynamicQueryService;

	/**
	 * 项目列表
	 */
	@ApiOperation(value = "项目-活动-列表")
	@ApiOperationSupport(order = 1)
	@RequestMapping(value = "/listProjectAndType", method = RequestMethod.POST)
	public ResultSegment listProject(@RequestParam(value = "proj", required = false) String proj) {
		List<Project> list = new ArrayList<>();
		if (StrUtil.isNotEmpty(proj)) {
			list = projectService.list(new QueryWrapper<Project>().eq("proj_code", proj));
		} else {
			list = projectService.list(new QueryWrapper<Project>());
		}
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, list);
	}

	/**
	 * 维度（事件）-列表
	 */
	@ApiOperation(value = "指标-列表")
	@ApiOperationSupport(order = 2)
	@RequestMapping(value = "/listCountOrDistion", method = RequestMethod.POST)
	public ResultSegment listCountOrDistion(@RequestParam(value = "proj", required = false) String projectId) {
		if (StrUtil.isBlank(projectId)) {
			return new ResultSegment(true, Constants.FAIL_CODE, Constants.FAIL_MSG, "参数异常");
		}
		List<EventVO> eventlist = eventService.getEventList(projectId);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, eventlist);
	}

	/**
	 * 
	 * @return
	 */
	@ApiOperation(value = "维度属性值 - 列表")
	@ApiOperationSupport(order = 5)
	@RequestMapping(value = "/listPropValues", method = RequestMethod.POST)
	public ResultSegment listPropValues(String dicTable, String joinField, String targetField, String orderField, Integer limit) {
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, dynamicQueryService.queryList(dicTable, joinField, targetField, orderField, limit));
	}
	
	/**
	 * 维度（事件）-列表
	 */
	@ApiOperation(value = "维度(事件属性)-列表&&公共过滤条件列表")
	@ApiOperationSupport(order = 3)
	@RequestMapping(value = "/listEvent", method = RequestMethod.POST)
	public ResultSegment listEvent(
			@RequestParam(value = "projectId", required = true)  Long projectId,
			@RequestParam(value = "eventIds", required = false) List<Long> eventIds) {
		List<ProjectEventMap> projectEventList = projectEventMapService.list(Wrappers.<ProjectEventMap>lambdaQuery().eq(ProjectEventMap::getProjectId, projectId).in(ProjectEventMap::getEventId, eventIds));
		List<Long> propIdList = new ArrayList<Long>();
		if(projectEventList != null && !projectEventList.isEmpty()) {
			for(ProjectEventMap prjEventMap : projectEventList) {
				if(StrUtil.isNotBlank(prjEventMap.getPropIdJson())){
					List<Long> pidlist = JSON.parseArray(prjEventMap.getPropIdJson(), Long.class);
					if(propIdList.isEmpty() || pidlist.isEmpty()) {
						propIdList.addAll(pidlist);
					}else {
						propIdList = propIdList.stream().filter(pidlist::contains).collect(Collectors.toList());
					}
				}
			}
		}
		List<Prop> list = propService.list(new QueryWrapper<Prop>().in(propIdList.size() > 0, "id", propIdList)
				.or(a -> a.eq("is_common", 1)));
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, list);
	}
	
	/**
	 * 留存分析模型 clickhouse
	 */
	@ApiOperation(value = "留存分析模型clickhouse")
	@ApiOperationSupport(order = 6)
	@RequestMapping(value = "/clickhouse/remainAnalysis", method = RequestMethod.POST)
	public ResultSegment clickhouseRemainremainAnalysis(@RequestBody RequestRetentionAnalysis requestRemainAnalysis)
			throws ParseException {
		JSONObject result = userEventService.reaminAnalysis(requestRemainAnalysis);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 漏斗分析模型 clickhouse
	 * @param requestHopperAnalysis
	 * @return
	 * @throws ParseException
	 */
	@ApiOperation(value = "转化漏斗分析模型")
	@ApiOperationSupport(order = 7)
	@RequestMapping(value = "/clickhouse/hopperAnalysis", method = RequestMethod.POST)
	public ResultSegment HopperAnalysis(@RequestBody RequestHopperAnalysis requestHopperAnalysis)
			throws ParseException {
		JSONObject result = userEventService.hopperAnalysis(requestHopperAnalysis);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 事件分析模型 clickhouse
	 */
	@ApiOperation(value = "事件分析模型clickhouse")
	@ApiOperationSupport(order = 5)
	@RequestMapping(value = "/clickhouse/eventAnalysis", method = RequestMethod.POST)
	public ResultSegment clickhouseEventAnalysis(@RequestBody RequestEventAnalysis eventAnalysisRequest)
			throws ParseException {
		if (eventAnalysisRequest.getStartDate() == null || eventAnalysisRequest.getEndDate() == null) {
			return new ResultSegment(false, 2000, "起止日期为必填参数");
		}
		JSONObject result = userEventService.eventAnalysis(eventAnalysisRequest);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 分布分析模型 clickhouse
	 */
	@ApiOperation(value = "分布分析模型clickhouse")
	@ApiOperationSupport(order = 8)
	@RequestMapping(value = "/clickhouse/distributeAnalysis", method = RequestMethod.POST)
	public ResultSegment distributeAnalysis(@RequestBody RequestDistributeAnalysis distributeAnalysisRequest)
			throws ParseException {
		JSONObject returnResult = userEventService.distributeAnalysis(distributeAnalysisRequest);
		if (returnResult != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, returnResult);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

}
