package org.springblade.useranalysis.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.sql.Query;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.useranalysis.dto.*;
import org.springblade.useranalysis.dto.param.BuriedPointAnalysisParam;
import org.springblade.useranalysis.service.BuriedPointAnalysisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.List;
import java.util.Map;

/**
 * @author hepingguo
 * @version 1.0
 * @description: TODO
 * @date 2023/11/25 13:21
 */
@RestController
@RequestMapping("/buried/point/analysis")
@Api(value = "埋点分析", tags = "埋点分析接口")
public class BuriedPointAnalysisController {

    @Autowired
    private UserClient userClient;

    @Autowired
    private TenantClient tenantClient;

    @Autowired
    private BuriedPointAnalysisService buriedPointAnalysisService;

    /**
     * 查询租户信息
     */
    @GetMapping("/tenant/list")
    @ApiOperation(value = "查询租户信息", notes = "tenantId为空则查询全量数据，不为空则按照租户id查询数据")
    public R<List<TenantModel>> getTenantList(@RequestParam(required = false) String tenantId) {

        return R.data(tenantClient.getTenantList(tenantId));
    }

    /**
     * 根据租户id查询用户信息
     */
    @GetMapping("/user/list")
    @ApiOperation(value = "通过租户id查询用户信息")
    public R<List<UserModel>> getUserByTenantId(@RequestParam String tenantId) {

        return R.data(userClient.listByTenantId(tenantId));
    }

    /**
     * 查询看板浏览次数top10
     */
    @GetMapping("/kanban/top")
    @ApiOperation(value = "查看看板浏览次数top10")
    public R<Map<String,List<NumberOfKanbanViewsDto>>> getKanbanViewsTop10(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getKanbanTop10(param));
    }

    /**
     * 查询活跃租户/用户top10
     */
    @GetMapping("/active/top")
    @ApiOperation(value = "查询活跃租户/用户top10")
    public R<Map<String,List<ActiveTenantUserDto>>> getActiveTenantOrUserTop10(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getActiveTenantOrUserTop(param));
    }


    /**
     *
     * 查询数据概况
     */
    @GetMapping("/data/overview")
    @ApiOperation(value = "查询数据概况")
    public R<Map<String,List<DataOverviewDto>>> getDataOverview(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getDataOverview(param));
    }

    /**
     * 看板类型浏览次数占比
     */
    @GetMapping("/kanban/pv")
    @ApiOperation(value = "看板类型浏览次数占比")
    public R<Map<String,List<KanbanTypeDto>>> getKanbanPv(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getKanbanPv(param));
    }

    /**
     * 新建看板浏览数据
     */
    @GetMapping("/kanban/new")
    @ApiOperation(value = "新建看板浏览数据")
    public R<Map<String,List<NewKanbanViewDto>>> getNewKanbanView(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getKanbanNewView(param));
    }

    /**
     * 明细数据
     */
    @GetMapping("/detail/data")
    @ApiOperation(value = "明细数据")
    public R<Page<DetailDataDto>> getDetailDataList(BuriedPointAnalysisParam param, Integer size, Integer page) {
        if (size == null) {
            size = 10;
        }
        if (page == null) {
            page = 1;
        }
        return R.data(buriedPointAnalysisService.getDetails(param,size,page));
    }

    /**
     * 查询页面分类
     */
    @GetMapping("/page/type")
    @ApiOperation(value = "查询页面分类")
    public R<List<DetailDataDto>> getPageType(BuriedPointAnalysisParam param) {

        return R.data(buriedPointAnalysisService.getPageTypes(param));
    }

    /**
     * 导出数据
     */
    @GetMapping("/download")
    @ApiOperation(value = "导出数据", notes = "传入参数与明细数据接口一致")
    public void download(BuriedPointAnalysisParam param, HttpServletResponse response) throws Exception{

        if (param == null) {
           param = new BuriedPointAnalysisParam();
        }
        if (param.getSize() == null) {
            param.setSize(1000);
        }
        if (param.getPage() == null) {
            param.setPage(1);
        }
        this.setExcelResponseProp(response, "数据明细");
        Page<DetailDataDto> details = buriedPointAnalysisService.getDetails(param, param.getSize(), param.getPage());
        long total = details.getTotal();
        if (total == 0) {
            throw new RuntimeException("查询无数据");
        }
        ExcelWriter build = EasyExcel.write(response.getOutputStream(), DetailDataDto.class).build();
        int page = 1;
        do {
            WriteSheet writeSheet = EasyExcel.writerSheet(page-1,"sheet"+page).build();
            param.setPage(page);
            details = buriedPointAnalysisService.getDetails(param, param.getSize(), param.getPage());
            build.write(details.getRecords(),writeSheet);
            page += 1;
        } while ((page - 1) * param.getSize() < total);
        build.finish();
    }

    private void setExcelResponseProp(HttpServletResponse response, String rawFileName) throws UnsupportedEncodingException {
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode(rawFileName, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
    }
}
