package org.springblade.useranalysis.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.ibatis.annotations.Param;
import org.springblade.core.tool.api.R;
import org.springblade.system.client.model.TenantModel;
import org.springblade.useranalysis.dto.VisualDataLineChartDto;
import org.springblade.useranalysis.dto.VisualDataPageDto;
import org.springblade.useranalysis.dto.VisualDataTopDto;
import org.springblade.useranalysis.dto.param.VisualDataAnalysisPageParams;
import org.springblade.useranalysis.service.VisualDataAnalysisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/analysis/visualData")
@Api(tags = "大屏数据统计")
public class VisualDataAnalysisController {
    @Autowired
    private VisualDataAnalysisService visualDataAnalysisService;

    /**
     * 查询明细数据-分页
     */
    @GetMapping("/page")
    @ApiOperation("查询明细数据-分页")
    public R<Page<VisualDataPageDto>> page(VisualDataAnalysisPageParams param) {
        return R.data(visualDataAnalysisService.page(param));
    }

    @GetMapping("/lineChart")
    @ApiOperation("大屏折线图")
    public R<List<VisualDataLineChartDto>> lineChart(VisualDataAnalysisPageParams param) {
        return R.data(visualDataAnalysisService.lineChart(param));
    }


    /**
     * 大屏top
     */
    @GetMapping("/getVisualTop")
    @ApiOperation("大屏top")
    public R<List<VisualDataTopDto>> getVisualTop(
            @ApiParam("可视化大屏 自助报表")
            @RequestParam(defaultValue = "可视化大屏") String type,
            @ApiParam("top多少 默认10")
            @RequestParam(defaultValue = "10") Integer top) {
        return R.data(visualDataAnalysisService.getVisualTop(type, top));
    }


}
