/**
 * 
 */
package org.springblade.useranalysis.enums;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

/**
 * @author wookvn
 *
 */
public enum EventFunction {

	eq{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.eq(column, values.get(0));
		}
	},
	
	ne{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.ne(column, values.get(0));
		}
	},
	
	lt{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			if(!isCommon) {
				queryWrapper.apply("toInt64OrZero({"+column+"}) < {0}", values.get(0));
			}else {
				queryWrapper.lt(column, values.get(0));
			}
		}
	},
	
	le{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			if(!isCommon) {
				queryWrapper.apply("toInt64OrZero("+column+") <= {0}", values.get(0));
			}else {
				queryWrapper.le(column, values.get(0));
			}
			//queryWrapper.le(column, values.get(0));
		}
	},
	
	gt{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			if(!isCommon) {
				queryWrapper.apply("toInt64OrZero("+column+") > {0}", values.get(0));
			}else {
				queryWrapper.le(column, values.get(0));
			}
			//queryWrapper.gt(column, values.get(0));
		}
	},
	
	ge{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			if(!isCommon) {
				queryWrapper.apply("toInt64OrZero("+column+") >= {0}", values.get(0));
			}else {
				queryWrapper.ge(column, values.get(0));
			}
			//queryWrapper.ge(column, values.get(0));
		}
	},
	
	between{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			if(values.size() == 2) {
				if(!isCommon) {
					queryWrapper.apply("toInt64OrZero("+column+") between {0} and {1}", values.get(0), values.get(1));
				}else {
					queryWrapper.between(column, values.get(0), values.get(1));
				}
				//queryWrapper.between(column, values.get(0), values.get(1));
			}
		}
	},
	
	isNotNull{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.isNotNull(column);
		}
	},
	
	isNull{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.isNull(column);
		}
	},
	
	in{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.in(column, values);
		}
	},
	
	notIn{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.notIn(column, values);
		}
	},
	
	like{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.apply("multiSearchAnyUTF8("+column+", {0}) = 1", values);
		}
	},
	
	notLike{
		@Override
		public void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon) {
			queryWrapper.apply("multiSearchAnyUTF8("+column+", {0}) = 0", values);
		}
	};
	
	public abstract void buildQuery(QueryWrapper<?> queryWrapper, String column, List<Object> values, boolean isCommon);

}
