package org.springblade.useranalysis.mapper;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.useranalysis.dto.*;
import org.springblade.useranalysis.dto.param.BuriedPointAnalysisParam;

import java.util.List;

/**
 * @author hepingguo
 * @version 1.0
 * @description: TODO
 * @date 2023/11/27 14:52
 */
@DS("clickhouse")
@InterceptorIgnore(tenantLine ="true")
public interface BuriedPointAnalysisMapper {

    /**
     * 查询看板浏览top10
     */
    @Select({"<script>",
            "select attr1 type, _attr2 name , _uv uv, _pv pv from(",
            " select attr1, groupArray(10)(attr2) as _attr2, groupArray(10)(uv) as _uv, groupArray(10)(pv) as _pv from(",
            " SELECT attr1, attr2, count(DISTINCT attr3) as uv, count(1) as pv from ads.user_event",
            "<where>",
            "event = #{param.event} ",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            "</where>",
            " group by attr1, attr2",
            " order by pv desc)",
            " group by attr1",
            ") array join _attr2, _uv, _pv",
            " order by attr1, _pv desc",
            "</script>"
    })
    List<NumberOfKanbanViewsDto> getKanbanTop10(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询活跃用户或者租户top10
     */
    @Select({"<script>",
            "SELECT 'tenant' as user, attr6 as tenant, count(DISTINCT attr3) as uv, count(1) as pv, 'tenant' as type",
            " from ads.user_event",
            " where event = #{param.event} and attr6 is not null",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
//            "<if test='param.user != null and param.user != \"\"'>and attr4 = #{param.user}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            " group by attr6 order by uv desc limit 10",
            " union all",
            " select attr4 as user, attr6 as tenant, count(DISTINCT attr3) as uv, count(1) as pv, 'user' as type",
            " from ads.user_event",
            " where event = #{param.event} and attr6 is not null",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            " group by attr4, attr6 order by uv desc limit 10",
            "</script>"
    })
    List<ActiveTenantUserDto> getTenantAndUserTop(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询数据概况
     */
    @Select({"<script>",
            "select d_date as date, '' as name, 0 as count, count(DISTINCT attr3) as uv, count(1) as pv , 'brokenLine' as type",
            " from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            " group by d_date",
            " union all",
            " select now() as date,attr1 as name, uniqExact(attr2) as count, 0 as uv, 0 as pv, 'type' as type",
            " from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
//            "<if test='param.type != null and param.type != \"\"'> and attr1 = #{param.type}</if>",
            "</where>",
            " group by attr1",
            "</script>"
    })
    List<DataOverviewDto> getDataOverview(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询看板类型浏览次数占比
     */
    @Select({"<script>",
            "select attr1 as name, 'type' as type, count(1) as pv",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            "group by attr1 order by pv desc",
            "union all",
            "select attr2 as name, 'name' as type,  count(1) as pv",
            "from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            "group by attr2 order by pv desc limit 10",
            "</script>"
    })
    List<KanbanTypeDto> getKanbanPv(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 新建看板浏览数据
     */
    @Select({"<script>",
             " select d_date date, '' as name, 0 as count, count(1) as pv, count(DISTINCT attr3) as uv, 'brokenLine' as type",
            "from ads.user_event ue inner join (",
            "select a.attr2 attr2 from",
            "(select distinct attr2 from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where> ) a",
            "left join",
            "(select distinct attr2 from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date &lt; #{param.startTime} or d_date &gt; #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where> ) b",
            "on a.attr2 = b.attr2 where b.attr2 is null) aaa on ue.attr2 = aaa.attr2 group by d_date",
            "union all",
            "select now() as date,attr1 name, uniqExact(attr2) as count, 0 as pv, 0 as uv, 'type' as type",
            "from ads.user_event ue inner join(",
            "select a.attr2 from",
            "(select distinct attr2 from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where> ) a",
            "left join",
            "(select attr2 from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date &lt; #{param.startTime} or d_date &gt; #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where> ) b",
            "on a.attr2 = b.attr2 where b.attr2 is null) aaa on ue.attr2 = aaa.attr2 group by attr1",
            "</script>"
    })
    List<NewKanbanViewDto> getKanbanNewView(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询明细数据-分页
     */
    @Select({
            "<script>",
            "select d_date date, attr6 tenantId, attr4 user, attr1 pageType, attr2 pageClassify, attr5 pageName, url, count(DISTINCT attr3) as uv, count(1) as pv",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "<if test='param.moduleType != null and param.moduleType != \"\"'> and attr5 like #{param.moduleType}</if>",
            "<if test='param.name != null and param.name != \"\"'> and attr2 like #{param.name}</if>",
            "<if test='param.url != null and param.url != \"\"'> and url like #{param.url}</if>",
            "</where>",
            "group by d_date, attr6, attr4, attr1, attr2, attr5, url",
            "order by pv desc",
            "</script>"
    })
    Page<DetailDataDto> details(@Param("param") BuriedPointAnalysisParam param, Page<DetailDataDto> page);

    /**
     * 查询行业分类
     */
    @Select({
            "<script>",
            "select distinct  attr5 pageClassify",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            "group by attr5",
            "</script>"
    })
    List<DetailDataDto> pageTypes(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询行业名称
     */
    @Select({
            "<script>",
            "select distinct attr2 pageName",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.moduleType != null and param.moduleType != \"\"'> and attr2 like #{param.moduleType}</if>",
            "</where>",
            "group by attr2",
            "</script>"
    })
    List<DetailDataDto> ModuleType(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询页面分类
     */
    @Select({
            "<script>",
            "select distinct attr1 pageType",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "</where>",
            "group by attr1",
            "</script>"
    })
    List<DetailDataDto> getFirstType(@Param("param") BuriedPointAnalysisParam param);

    @Select({"<script>",
            "select attr5 as name, 'type' as type, count(1) as pv",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' >and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            "group by attr5 order by pv desc",
            "union all",
            "select attr2 as name, 'name' as type,  count(1) as pv",
            "from ads.user_event",
            "<where>",
            "event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "</where>",
            "group by attr2 order by pv desc limit 10",
            "</script>"
    })
    List<KanbanTypeDto> getKanbanPvByType(@Param("param") BuriedPointAnalysisParam param);

    /**
     * 查询明细数据不分页
     */
    @Select({
            "<script>",
            "select d_date date, attr6 tenantId, attr4 user, attr1 pageType, attr2 pageClassify, attr5 pageName, url, count(DISTINCT attr3) as uv, count(1) as pv",
            "from ads.user_event",
            "<where>",
            " event = #{param.event}",
            "<if test='param.tenantId != null and param.tenantId != \"\"'> and attr6 = #{param.tenantId}</if>",
            "<if test='param.userId != null and param.userId > 0 '> and attr3 = #{param.userId}</if>",
            "<if test='param.startTime != null and param.startTime != \"\" and param.endTime != null and param.endTime != \"\"' > and d_date between #{param.startTime} and #{param.endTime}</if>",
            "<if test='param.type != null and param.type != \"\"'> and attr1 like #{param.type}</if>",
            "<if test='param.moduleType != null and param.moduleType != \"\"'> and attr5 like #{param.moduleType}</if>",
            "<if test='param.name != null and param.name != \"\"'> and attr2 like #{param.name}</if>",
            "<if test='param.url != null and param.url != \"\"'> and url like #{param.url}</if>",
            "</where>",
            "group by d_date, attr6, attr4, attr1, attr2, attr5, url",
            "order by pv desc",
            "</script>"
    })
    List<DetailDataDto> detailList(BuriedPointAnalysisParam param);
}
