package org.springblade.useranalysis.mapper;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.useranalysis.dto.RequestWebsite;
import org.springblade.useranalysis.entity.UserEvent;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;

/**
 * (UserEvent)表数据库访问层
 *
 * @author limeng
 * @since 2021-07-06 14:38:03
 */
@InterceptorIgnore(tenantLine = "true")
@Mapper
public interface UserEventMapper extends BaseMapper<UserEvent> {

	@Select({
		"select toDate('${startDate}') as ds, ${selectColmns}",
		"from (",
		" with",
		"	d_date as dt,",
		"  	toDate('${startDate}') as firstDate",
		" SELECT",
		"   anonymous_id,", 
		"   retention(",
		"     ${retentionColmns}",
		"   ) AS r",
		"   from ads.user_event",
		"   ${ew.customSqlSegment}",
		"	group by anonymous_id",
		")"
	})
    List<Map<String, Object>> remainAnalysisData(@Param("startDate") String startDate,
                                                 @Param("selectColmns") String selectColmns,
                                                 @Param("retentionColmns") String retentionColmns,
                                                 @Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);

    @Select("select ${targets} dt, level_index, count(1) as count from" +
            "(" +
            "    select ${targets} dt, user_id, " +
            "        arrayWithConstant(level, 1) levels, " +
            "        arrayJoin(arrayEnumerate(levels)) level_index" +
            "      from (" +
            "		with " +
            "		  ${selectField},"+
            "		  d_timestamp as ds"+
            "        select " +
            "        ${targets} dt, anonymous_id as user_id," +
            "          windowFunnel(${windowSeconds})(" +
            "            ds  ${eventList} " +
            "          ) AS level" +
            "        from ads.user_event " +
            "        ${ew.customSqlSegment} " +
            "        group by ${targets} dt, user_id" +
            "    )" +
            ")" +
            "group by ${targets} dt, level_index " +
            "order by ${targets} dt, level_index")
    List<JSONObject> hopperAnalysis(@Param("targets") String targets,
                                    @Param("eventList") String eventList,
                                    @Param("windowSeconds") Long windowSeconds,
                                    @Param("selectField") String selectField,
                                    @Param(Constants.WRAPPER) QueryWrapper<UserEvent> TagWrapper);

    @Select({
    	"<script>",
		"select ${target} ${countSql} from(",
		"with d_date as dt",
		"select ${target} anonymous_id, count(1) as cnt",
		"from ads.user_event",
		"${ew.customSqlSegment}",
		"group by ${target} anonymous_id",
		") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
	List<JSONObject> distributeAnalysis(@Param("target") String target, 
										@Param("targetWithoutComma") String targetWithoutComma,
										@Param("countSql") String countSql,
										@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select({
    	"<script>",
    	"select ${target} ${countSql} from(",
    	"	with toHour(d_timestamp) as hours, d_date as dt",
    	" select ${target} hours, uniqCombined(anonymous_id) as num",
    	" from ads.user_event",
    	" ${ew.customSqlSegment}",
    	" group by ${target} hours",
    	") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
    List<JSONObject> distributeAnalysisOfTime(@Param("target") String target, 
											@Param("targetWithoutComma") String targetWithoutComma,
											@Param("countSql") String countSql,
											@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select({
    	"<script>",
		"select ${target} ${countSql} from(",
		"with d_date as dt",
		"select ${target} uniqCombined(${attrCode}) as cnt",
		"from ads.user_event",
		"${ew.customSqlSegment}",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		") ",
		"<if test='targetWithoutComma != null and targetWithoutComma != \"\"'>group by ${targetWithoutComma}</if>",
		"</script>"
    })
    List<JSONObject> distributeAnalysisOfUniqAttr(
    							@Param("attrCode") String attrCode,
    							@Param("target") String target, 
								@Param("targetWithoutComma") String targetWithoutComma,
								@Param("countSql") String countSql,
								@Param(Constants.WRAPPER) QueryWrapper<UserEvent> queryWrapper);
    
    @Select("select aid from ads.user_event where ptype = #{value} and aid is not null group by aid")
    List<String> findAidList(String ptype);
    
    @Select({"<script>",
    	"select DISTINCT title,url from ads.user_event where title is not null and title != ''",
    	"<if test='event != null and event != \"\"'> and event = #{event}</if>",
    	"</script>"})
    List<Map<String, Object>> findTitleList(@Param("event") String event);
    
    @Select({"<script>",
    	"select * from user_event",
    	"<where>",
    	"<if test='requestWebsite.proj != null and requestWebsite.proj != \"\"'> and proj = #{requestWebsite.proj}</if>",
    	"<if test='requestWebsite.event != null and requestWebsite.event != \"\"'> and event = #{requestWebsite.event}</if>",
    	"<if test='requestWebsite.title != null and requestWebsite.title != \"\"'> and title = #{requestWebsite.title}</if>",
    	"<if test='requestWebsite.platformType != null and requestWebsite.platformType != \"\"'> and platform_type = #{requestWebsite.platformType}</if>",
    	"<if test='requestWebsite.distinctId != null and requestWebsite.distinctId != \"\"'> and distinct_id = #{requestWebsite.distinctId}</if>",
    	"<if test='requestWebsite.startTime != null and requestWebsite.startTime != \"\"'> and d_timestamp &gt;= #{requestWebsite.startTime}</if>",
    	"<if test='requestWebsite.endTime != null and requestWebsite.endTime != \"\"'> and d_timestamp &lt;= #{requestWebsite.endTime}</if>",
    	"</where>",
    	"limit ${requestWebsite.limit}",
    	"</script>"})
    List<Map<String, Object>> findEventList(@Param("requestWebsite")RequestWebsite requestWebsite);
}
