package org.springblade.useranalysis.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.pson.entity.Event;
import org.springblade.pson.service.IEventService;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.useranalysis.dto.*;
import org.springblade.useranalysis.dto.param.AccessMonitorPageParams;
import org.springblade.useranalysis.dto.param.VisualDataAnalysisPageParams;
import org.springblade.useranalysis.entity.UserEvent;
import org.springblade.useranalysis.mapper.AccessMonitorMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class AccessMonitorService {

    @Autowired
    private AccessMonitorMapper accessMonitorMapper;
    @Autowired
    private IEventService eventService;
    @Autowired
    private UserClient userClient;

    /**
     * 查询明细数据-分页
     */
    public Page<AccessMonitorPageDto> page(AccessMonitorPageParams param) {
        Page<UserEvent> page = accessMonitorMapper.page(param, new Page(param.getPage(), param.getSize()));
        Page<AccessMonitorPageDto> customPage = new Page<>();
        customPage.setCurrent(page.getCurrent());
        customPage.setSize(page.getSize());
        customPage.setTotal(page.getTotal());
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> eventCodes = page.getRecords().stream().map(UserEvent::getEvent).distinct().collect(Collectors.toList());
            List<String> userIds = page.getRecords().stream().map(UserEvent::getLoginId).distinct().collect(Collectors.toList());
            List<Event> events = eventService.lambdaQuery().in(Event::getCode,eventCodes).list();
            Map<String, String> eventMap = events.stream().collect(Collectors.toMap(Event::getCode, Event::getName));
            ArrayList<AccessMonitorPageDto> accessMonitorPageDtos = new ArrayList<>();
            for (UserEvent record : page.getRecords()) {
                AccessMonitorPageDto accessMonitorPageDto = new AccessMonitorPageDto();
                BeanUtils.copyProperties(record, accessMonitorPageDto);
                accessMonitorPageDto.setEventName(eventMap.get(record.getEvent()));
                accessMonitorPageDtos.add(accessMonitorPageDto);
            }
            customPage.setRecords(accessMonitorPageDtos);
        }
        return customPage;
    }


    public List<AccessMonitorLineChartDto> lineChart(AccessMonitorPageParams param) {
        return accessMonitorMapper.lineChart(param);
    }


    /**
     * 查询看板浏览top10
     */
    public List<AccessMonitorTopDto> getAccessMonitorTop(Integer top) {
        return accessMonitorMapper.getAccessMonitorTop(top);
    }

}
