package org.springblade.useranalysis.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.SneakyThrows;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.useranalysis.dto.*;
import org.springblade.useranalysis.dto.param.DataSourceAnalysisDetailsPageParams;
import org.springblade.useranalysis.dto.param.DataSourceAnalysisPageParams;
import org.springblade.useranalysis.dto.param.VisualDataAnalysisPageParams;
import org.springblade.useranalysis.entity.UserEvent;
import org.springblade.useranalysis.mapper.BuriedPointAnalysisMapper;
import org.springblade.useranalysis.mapper.DataSourceAnalysisMapper;
import org.springblade.useranalysis.mapper.UserEventMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class DataSourceAnalysisService {
    @Autowired
    private DataSourceAnalysisMapper dataSourceAnalysisMapper;
    @Autowired
    private TenantClient tenantClient;

    /**
     * 数据源监控明细
     */
    @SneakyThrows
    public Page<DataSourceDetailsPageDto> detailsPage(DataSourceAnalysisDetailsPageParams params) {
        Page<DataSourceDetailsPageDto> page = dataSourceAnalysisMapper.detailsPage(params, new Page(params.getPage(), params.getSize()));
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> tenantIds = page.getRecords().stream().map(DataSourceDetailsPageDto::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            page.getRecords().forEach(detail -> tenants.forEach(tenant -> {
                if (StrUtil.isNotEmpty(detail.getTenantId()) && tenant.getTenantId().equals(detail.getTenantId())) {
                    detail.setTenant(tenant.getTenantName());
                }
            }));
        }
        return page;
    }

    @SneakyThrows
    public Page<DataSourcePageDto> page(DataSourceAnalysisPageParams params) {
        Page<DataSourcePageDto> page = dataSourceAnalysisMapper.page(params, new Page(params.getPage(), params.getSize()));
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> tenantIds = page.getRecords().stream().map(DataSourcePageDto::getTenantId).collect(Collectors.toList());
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            page.getRecords().forEach(detail -> tenants.forEach(tenant -> {
                if (StrUtil.isNotEmpty(detail.getTenantId()) && tenant.getTenantId().equals(detail.getTenantId())) {
                    detail.setTenant(tenant.getTenantName());
                }
            }));
        }
        return page;
    }

    public List<DataSourceLineChartDto> lineChart(DataSourceAnalysisPageParams param) {
        return dataSourceAnalysisMapper.lineChart(param);
    }

    public List<String> responsiblePersons() {
        return dataSourceAnalysisMapper.responsiblePersons();
    }
}
