package org.springblade.useranalysis.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.SneakyThrows;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.bi.client.VisualDataClient;
import org.springblade.bi.client.model.VisualDataModel;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.UserClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.system.client.model.UserModel;
import org.springblade.useranalysis.dto.DetailDataDto;
import org.springblade.useranalysis.dto.VisualDataLineChartDto;
import org.springblade.useranalysis.dto.VisualDataPageDto;
import org.springblade.useranalysis.dto.VisualDataTopDto;
import org.springblade.useranalysis.dto.param.DataSourceAnalysisPageParams;
import org.springblade.useranalysis.dto.param.VisualDataAnalysisPageParams;
import org.springblade.useranalysis.entity.UserEvent;
import org.springblade.useranalysis.mapper.UserEventMapper;
import org.springblade.useranalysis.mapper.VisualDataAnalysisMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class VisualDataAnalysisService {

    @Autowired
    private VisualDataAnalysisMapper visualDataAnalysisMapper;
    @Autowired
    private TenantClient tenantClient;
    @Autowired
    private VisualDataClient visualDataClient;
    @Autowired
    private UserClient userClient;

    /**
     * 查询明细数据-分页
     */
    public Page<VisualDataPageDto> page(VisualDataAnalysisPageParams param) {
        Page<VisualDataPageDto> page = visualDataAnalysisMapper.page(param, new Page(param.getPage(), param.getSize()));
        if (CollectionUtil.isNotEmpty(page.getRecords())) {
            List<String> tenantIds = page.getRecords().stream().map(VisualDataPageDto::getTenantId).collect(Collectors.toList());
            List<String> visualIds = page.getRecords().stream().map(VisualDataPageDto::getVisualId).collect(Collectors.toList());
            List<VisualDataModel> visualDataModels = visualDataClient.listByIds(visualIds);
            Map<Integer, Long> visualMap = visualDataModels.stream().collect(Collectors.toMap(VisualDataModel::getId, VisualDataModel::getCreateUser));
            List<Long> userIds = visualDataModels.stream().map(VisualDataModel::getCreateUser).collect(Collectors.toList());
            List<UserModel> userModels = userClient.listByIds(userIds);
            Map<Long, String> userMap = userModels.stream().collect(Collectors.toMap(UserModel::getId, UserModel::getRealName));
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            Map<String, String> teanntMap = tenants.stream().collect(Collectors.toMap(TenantModel::getTenantId, TenantModel::getTenantName));
            page.getRecords().forEach(detail -> {
                detail.setTenant(teanntMap.get(detail.getTenantId()));
                detail.setCreatorName(userMap.get(visualMap.get(detail.getVisualId())));
            });
        }
        return page;
    }


    public List<VisualDataLineChartDto> lineChart(VisualDataAnalysisPageParams param) {
        return visualDataAnalysisMapper.lineChart(param);
    }


    /**
     * 查询看板浏览top10
     */
    public List<VisualDataTopDto> getVisualTop(String type, Integer top) {
        return visualDataAnalysisMapper.getVisualTop(type, top);
    }
}
