package org.springblade.useranalysis.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springblade.system.client.TenantClient;
import org.springblade.system.client.model.TenantModel;
import org.springblade.useranalysis.dto.*;
import org.springblade.useranalysis.dto.param.BuriedPointAnalysisParam;
import org.springblade.useranalysis.mapper.BuriedPointAnalysisMapper;
import org.springblade.useranalysis.service.BuriedPointAnalysisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hepingguo
 * @version 1.0
 * @description: TODO
 * @date 2023/11/27 14:51
 */
@Service
public class BuriedPointAnalysisServiceImpl implements BuriedPointAnalysisService {

    @Autowired
    private BuriedPointAnalysisMapper buriedPointAnalysisMapper;

    @Autowired
    private TenantClient tenantClient;

    @Value("${blade.file.exportComponent}")
    private String exportComponentDir;

    @Override
    public Map<String, List<NumberOfKanbanViewsDto>> getKanbanTop10(BuriedPointAnalysisParam param) {
        param = getParam(param);
        Map<String, List<NumberOfKanbanViewsDto>> map = new HashMap<>();
        List<NumberOfKanbanViewsDto> list = buriedPointAnalysisMapper.getKanbanTop10(param);
        if (CollectionUtil.isNotEmpty(list)) {
            Map<String, List<NumberOfKanbanViewsDto>> listMap = list.stream().collect(Collectors.groupingBy(NumberOfKanbanViewsDto::getType));
            if (CollectionUtil.isNotEmpty(listMap.get("模板"))) {
                List<NumberOfKanbanViewsDto> templates = listMap.get("模板").stream().sorted(Comparator.comparing(NumberOfKanbanViewsDto::getPv).reversed()).collect(Collectors.toList());
                map.put("模板",templates);
            }
            if (CollectionUtil.isNotEmpty(listMap.get("幻灯片"))) {
                List<NumberOfKanbanViewsDto> templates = listMap.get("幻灯片").stream().sorted(Comparator.comparing(NumberOfKanbanViewsDto::getPv).reversed()).collect(Collectors.toList());
                map.put("幻灯片",templates);
            }
            if (CollectionUtil.isNotEmpty(listMap.get("可视化大屏"))) {
                List<NumberOfKanbanViewsDto> templates = listMap.get("可视化大屏").stream().sorted(Comparator.comparing(NumberOfKanbanViewsDto::getPv).reversed()).collect(Collectors.toList());
                map.put("可视化大屏",templates);
            }
            if (CollectionUtil.isNotEmpty(listMap.get("自助报表"))) {
                List<NumberOfKanbanViewsDto> templates = listMap.get("自助报表").stream().sorted(Comparator.comparing(NumberOfKanbanViewsDto::getPv).reversed()).collect(Collectors.toList());
                map.put("自助报表",templates);
            }
        }

        return map;
    }

    @Override
    public Map<String, List<ActiveTenantUserDto>> getActiveTenantOrUserTop(BuriedPointAnalysisParam param) {
        Map<String,List<ActiveTenantUserDto>> map = new HashMap<>();
        param = getParam(param);
        List<ActiveTenantUserDto> activeTenants = buriedPointAnalysisMapper.getTenantAndUserTop(param);
        // 如果结果不为空则查询租户信息，并封装结果
        if (CollectionUtil.isNotEmpty(activeTenants)) {
            List<String> tenantIds = activeTenants.stream().map(ActiveTenantUserDto::getTenant).collect(Collectors.toList());
            // 根据租户id集合查询租户信息集合
            List<TenantModel> tenantByTenantIds = tenantClient.getTenantByTenantIds(tenantIds);
            // 如果租户信息不为空则循环为结果的租户信息进行替换
            if (CollectionUtil.isNotEmpty(tenantByTenantIds)) {
                activeTenants.forEach(active -> {
                    for (TenantModel tenant : tenantByTenantIds) {
                        if (active.getTenant().equals(tenant.getTenantId())) {
                            active.setTenant(tenant.getTenantName());
                        }
                    }
                });
            }
            // 将查询结果按照类型分类
            Map<String, List<ActiveTenantUserDto>> collect = activeTenants.stream().collect(Collectors.groupingBy(ActiveTenantUserDto::getType));
            if (CollectionUtil.isNotEmpty(collect.get("user"))) {
                List<ActiveTenantUserDto> user = collect.get("user").stream().sorted(Comparator.comparing(ActiveTenantUserDto::getUv).reversed()).collect(Collectors.toList());
                map.put("user",user);
            }
            if (CollectionUtil.isNotEmpty(collect.get("tenant"))) {
                List<ActiveTenantUserDto> tenant = collect.get("tenant").stream().sorted(Comparator.comparing(ActiveTenantUserDto::getUv).reversed()).collect(Collectors.toList());
                map.put("tenant",tenant);
            }
        }
        return map;
    }

    @Override
    public Map<String, List<DataOverviewDto>> getDataOverview(BuriedPointAnalysisParam param) {
        param = getParam(param);
        List<DataOverviewDto> list = buriedPointAnalysisMapper.getDataOverview(param);

        if (CollectionUtil.isNotEmpty(list)) {
            return list.stream().collect(Collectors.groupingBy(DataOverviewDto::getType));
        }

        return null;
    }

    @Override
    public Map<String, List<KanbanTypeDto>> getKanbanPv(BuriedPointAnalysisParam param) {
        param = getParam(param);
        Map<String,List<KanbanTypeDto>> map = new HashMap<>();
        List<KanbanTypeDto> list = null;
        if (StrUtil.isBlank(param.getType())) {
            list = buriedPointAnalysisMapper.getKanbanPv(param);
        } else {
            list = buriedPointAnalysisMapper.getKanbanPvByType(param);
        }
        if (CollectionUtil.isNotEmpty(list)) {
            Map<String, List<KanbanTypeDto>> collect = list.stream().collect(Collectors.groupingBy(KanbanTypeDto::getType));
            if (CollectionUtil.isNotEmpty(collect.get("type"))) {
                List<KanbanTypeDto> typeList = collect.get("type");
                double count = typeList.stream().mapToDouble(KanbanTypeDto::getPv).sum();
                typeList.forEach(type -> type.setPercentage(type.getPv() / count));
                map.put("type",typeList);
            }
            if (CollectionUtil.isNotEmpty(collect.get("name"))) {
                List<KanbanTypeDto> nameList = collect.get("name");
                double count = nameList.stream().mapToDouble(KanbanTypeDto::getPv).sum();
                nameList.forEach(name -> name.setPercentage(name.getPv() / count));
                map.put("name",nameList);
            }
        }

        return map;
    }

    @Override
    public Map<String, List<NewKanbanViewDto>> getKanbanNewView(BuriedPointAnalysisParam param) {
        param = getParam(param);
        List<NewKanbanViewDto> list = buriedPointAnalysisMapper.getKanbanNewView(param);
        if (CollectionUtil.isNotEmpty(list)) {
            return list.stream().collect(Collectors.groupingBy(NewKanbanViewDto::getType));
        }
        return null;
    }

    @Override
    public Page<DetailDataDto> getDetails(BuriedPointAnalysisParam param,Integer size, Integer page) {
        param = getParam(param);
        Page<DetailDataDto> pages = new Page<>(page,size);
        Page<DetailDataDto> details = buriedPointAnalysisMapper.details(param,pages);
        List<String> tenantIds = new ArrayList<>();
        if (details!=null && CollectionUtil.isNotEmpty(details.getRecords())) {
            for (DetailDataDto detail : details.getRecords()) {
                if (StrUtil.isNotEmpty(detail.getTenantId())) {
                    tenantIds.add(detail.getTenantId());
                }
            }
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            details.getRecords().forEach(detail -> tenants.forEach(tenant -> {
                if (StrUtil.isNotEmpty(detail.getTenantId()) && tenant.getTenantId().equals(detail.getTenantId())) {
                    detail.setTenant(tenant.getTenantName());
                }
            }));

            return details;
        }

        return null;
    }

    @Override
    public List<DetailDataDto> getPageTypes(BuriedPointAnalysisParam param) {
        param = getParam(param);
        if (StrUtil.isNotEmpty(param.getType())) {
            return buriedPointAnalysisMapper.pageTypes(param);
        } else if (StrUtil.isNotEmpty(param.getModuleType())) {
            return buriedPointAnalysisMapper.ModuleType(param);
        } else {
            return buriedPointAnalysisMapper.getFirstType(param);
        }
    }

    @Override
    public File exportExcel(BuriedPointAnalysisParam param) {
        param = getParam(param);
        String fileName = "明细数据";
        List<DetailDataDto> details = buriedPointAnalysisMapper.detailList(param);
        List<String> tenantIds = new ArrayList<>();
        if (details!=null && CollectionUtil.isNotEmpty(details)) {
            for (DetailDataDto detail : details) {
                if (StrUtil.isNotEmpty(detail.getTenantId())) {
                    tenantIds.add(detail.getTenantId());
                }
            }
            List<TenantModel> tenants = tenantClient.getTenantByTenantIds(tenantIds);
            details.forEach(detail -> tenants.forEach(tenant -> {
                if (StrUtil.isNotEmpty(detail.getTenantId()) && tenant.getTenantId().equals(detail.getTenantId())) {
                    detail.setTenant(tenant.getTenantName());
                }
            }));
        }
        return null;
    }

    public BuriedPointAnalysisParam getParam(BuriedPointAnalysisParam param){
        if (StrUtil.isNotBlank(param.getType())) {
            param.setType("%" + param.getType() + "%");
        }
        if (StrUtil.isNotBlank(param.getModuleType())) {
            param.setModuleType("%" + param.getModuleType() + "%");
        }
        if (StrUtil.isNotBlank(param.getName())) {
            param.setName("%" + param.getName() + "%");
        }
        if (StrUtil.isNotBlank(param.getUrl())) {
            param.setUrl("%" + param.getUrl() + "%");
        }
        return param;
    }
}
