package org.springblade.useranalysis.service.impl;

import java.text.ParseException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springblade.pson.entity.Event;
import org.springblade.pson.entity.EventPropMap;
import org.springblade.pson.entity.Prop;
import org.springblade.pson.service.IEventPropMapService;
import org.springblade.pson.service.IEventService;
import org.springblade.pson.service.IPropService;
import org.springblade.useranalysis.common.utils.Tuple;
import org.springblade.useranalysis.dto.BaseEventCode;
import org.springblade.useranalysis.dto.EventAnalysisCode;
import org.springblade.useranalysis.dto.EventDistributeCode;
import org.springblade.useranalysis.dto.EventDistributeRange;
import org.springblade.useranalysis.dto.EventFilterCondition;
import org.springblade.useranalysis.dto.RequestDistributeAnalysis;
import org.springblade.useranalysis.dto.RequestEventAnalysis;
import org.springblade.useranalysis.dto.RequestHopperAnalysis;
import org.springblade.useranalysis.dto.RequestRetentionAnalysis;
import org.springblade.useranalysis.dto.RequestWebsite;
import org.springblade.useranalysis.dto.param.ParamProp;
import org.springblade.useranalysis.dto.param.ParamUnit;
import org.springblade.useranalysis.entity.UserEvent;
import org.springblade.useranalysis.enums.DistributeEventUnit;
import org.springblade.useranalysis.enums.HopperDateType;
import org.springblade.useranalysis.enums.RetentionEventUnit;
import org.springblade.useranalysis.mapper.UserEventMapper;
import org.springblade.useranalysis.service.UserEventService;
import org.springblade.useranalysis.service.normal.DynamicQueryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * (UserEvent)表服务实现类
 *
 * @author limeng
 * @since 2021-07-06 14:38:03
 */
@Service("userEventService")
@DS("clickhouse")
public class UserEventServiceImpl extends ServiceImpl<UserEventMapper, UserEvent> implements UserEventService {

	@Autowired
	private UserEventMapper baseMapper;
	
	@Autowired
	private IEventService eventService;

	@Autowired
	private IPropService propService;

	@Autowired
	private IEventPropMapService eventPropMapService;
	
	@Autowired
	private DynamicQueryService dynamicQueryService;

	/**
	 * 留存分析
	 */
	@Override
	public JSONObject reaminAnalysis(RequestRetentionAnalysis requestRemainAnalysis) throws ParseException {
		JSONArray detailList = new JSONArray();
		JSONObject returnObject = new JSONObject();
		
		if(requestRemainAnalysis.getUnit() == null) { //设置默认周期
			requestRemainAnalysis.setUnit(RetentionEventUnit.day);
			requestRemainAnalysis.setDuration(7);
		}
		
		DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(RequestRetentionAnalysis.DATE_FORMAT);
		LocalDate startDate = LocalDate.parse(requestRemainAnalysis.getStartDate(), dateTimeFormatter);
		LocalDate endDate = LocalDate.parse(requestRemainAnalysis.getEndDate(), dateTimeFormatter);
		ParamUnit paramUnit = requestRemainAnalysis.getUnit().calculate(startDate, endDate, RequestRetentionAnalysis.DATE_FORMAT);
		
		String firstEventCode = requestRemainAnalysis.getFirstEvent().getEventCode();
		String secondEventCode = requestRemainAnalysis.getSecondEvent().getEventCode();
		
		//只根据firstEventCode查询
		Map<String, List<ParamProp>> eventPropMap = initEventProp(Arrays.asList(firstEventCode), requestRemainAnalysis.getFilterParameter().getConditions(), null);
		
		for(String dateStr : paramUnit.getDateStartList()) {
			//初始行为用户数
			StringBuilder selectColmns = new StringBuilder("sum(r[1]) AS activeAccountNum");
			StringBuilder retentionColmns = new StringBuilder(String.format("event='%s' and dt = firstDate", firstEventCode));
			
			//当日用户数，第1日用户数，第2日用户数，依次类推。 下标从2开始
			for(int i = 2; i < requestRemainAnalysis.getDuration() + 3; i++) {
				selectColmns.append(",sum(r[{num}]) as day{num}count, day{num}count / activeAccountNum AS day{num}ratio".replace("{num}", i+""));
				if(requestRemainAnalysis.getUnit() == RetentionEventUnit.day) {
					retentionColmns.append(String.format(",event='%s' ", secondEventCode))
										.append(String.format("and dt = firstDate + INTERVAL %s %s", i - 2, requestRemainAnalysis.getUnit().clickHouseUnit));
				}else {
					retentionColmns.append(String.format(",event='%s' ", secondEventCode))
									.append(String.format("and dt > firstDate + INTERVAL %s %s and dt <= firstDate + INTERVAL %s %s",
											i - 2, requestRemainAnalysis.getUnit().clickHouseUnit, 
											i - 1, requestRemainAnalysis.getUnit().clickHouseUnit));
				}
			}
			
			QueryWrapper<UserEvent> queryWrapper = buildCondition(requestRemainAnalysis.getPtype(), requestRemainAnalysis.getFilterParameter(), eventPropMap.get(firstEventCode));
			queryWrapper.ge("dt", requestRemainAnalysis.getStartDate()).le("dt", requestRemainAnalysis.getEndDate());
			//获取数据
			List<Map<String, Object>> maps = baseMapper.remainAnalysisData(dateStr, selectColmns.toString(), retentionColmns.toString(), queryWrapper);
			
			JSONObject detail = new JSONObject();
			detail.put("dateValue", maps.get(0).get("ds"));
			detail.put("totalPeople", maps.get(0).get("activeAccountNum"));
			
			JSONArray subsArr = new JSONArray();
			
			for(int i = 2; i < requestRemainAnalysis.getDuration() + 3; i++) {
				JSONObject subs = new JSONObject();
				subs.put("people", maps.get(0).get("day" + i + "count"));
				//保留两位小数
				subs.put("percent", pernt( maps.get(0).get("day" + i + "ratio")));
				subsArr.add(subs);
			}
			detail.put("subs", subsArr);
			detailList.add(detail);
		}

		for(int i = 0; i < detailList.size(); i++) {
			JSONObject itemObject = detailList.getJSONObject(i);
			itemObject.getLongValue("totalPeople");
			itemObject.getJSONArray("subs");
		}
		
		returnObject.put("detailList", detailList);
		returnObject.put("dataList", paramUnit.getLabelList());
		return returnObject;
	}

	/**
	 * 漏斗分析
	 */
	@Override
	public JSONObject hopperAnalysis(RequestHopperAnalysis requestHopperAnalysis) throws ParseException {
		List<BaseEventCode> stepEventCodes = requestHopperAnalysis.getSteps();
		String eventCode = stepEventCodes.get(0).getEventCode();
		//获取第一个事件的查询维度和属性
		Map<String, List<ParamProp>> eventPropMap = initEventProp(Arrays.asList(eventCode), 
											requestHopperAnalysis.getFilterParameter().getConditions(), requestHopperAnalysis.getTarget());
		List<ParamProp> paramPropList = eventPropMap.get(eventCode);
		//target维度ID列表转code列表
//		List<String> targetPropCodeList = targetPropId2Code(requestHopperAnalysis.getTarget(), eventPropMap.get(eventCode));
		List<Tuple<String, Long>> targetPropCodeList = targetPropId2Code(requestHopperAnalysis.getTarget(), eventPropMap.get(eventCode));
		//获取 细分维度拼接字符串  A1,A2,
		StringBuilder colmusString = new StringBuilder();
		for (Tuple<String, Long> tuple : targetPropCodeList) {
			colmusString.append(tuple.getK()).append(",");
		}
		//拼接事件
		StringBuilder eventStringBuilder = new StringBuilder();
		stepEventCodes.stream().forEach(i -> {
			eventStringBuilder.append(",event = '" + i.getEventCode() + "'");
		});
		
		JSONObject dataObject = new JSONObject();
		dataObject.put("eventCodes", requestHopperAnalysis.getSteps().stream().map(BaseEventCode::getEventCode).collect(Collectors.toList()));
		dataObject.put("eventNames", requestHopperAnalysis.getSteps().stream().map(BaseEventCode::getName).collect(Collectors.toList()));


		QueryWrapper<UserEvent> queryWrapper = buildCondition(requestHopperAnalysis.getPtype(), requestHopperAnalysis.getFilterParameter(), eventPropMap.get(eventCode));
		queryWrapper.ge("d_date", requestHopperAnalysis.getStartDate()).le("d_date", requestHopperAnalysis.getEndDate());
		
		long windowSeconds = requestHopperAnalysis.getUnit() == null || requestHopperAnalysis.getDuration() == null ? 1800 : requestHopperAnalysis.getUnit().getSeconds(requestHopperAnalysis.getDuration());
		HopperDateType dateType = requestHopperAnalysis.getDateType() == null ? HopperDateType.day : requestHopperAnalysis.getDateType();
		List<JSONObject> dataList = baseMapper.hopperAnalysis(colmusString.toString(), eventStringBuilder.toString(), windowSeconds, dateType.value, queryWrapper);

		//保存<tableName, [columnValue]>
		Map<String, List<String>> tableSelectMap = new HashMap<String, List<String>>();
		// 细分属性->JsonObject
		Map<String, List<JSONObject>> wdMap = dataList.stream().collect(Collectors.groupingBy(e -> {
			List<String> keyList = new ArrayList<String>();
			for (Tuple<String, Long> targetPropCode : targetPropCodeList) {
				if(targetPropCode.getK() != null && e.get(targetPropCode.getK()) != null) {
					String columnValue = e.get(targetPropCode.getK()).toString();
					//如果存在需要查询动态表，占位符操作，最后替换
					Optional<ParamProp> optional = paramPropList.stream().filter(f -> f.getId().longValue() == targetPropCode.getV() && f.getDatatype() == 4).findAny();
					if(optional.isPresent()) {
						ParamProp paramProp = optional.get();
						keyList.add(String.format("{%s}", columnValue));
						String key = String.format("%s|%s|%s", paramProp.getDicTable(), paramProp.getJoinField(), paramProp.getTargetField());
						List<String> list = tableSelectMap.get(key);
						if(list == null) {
							list = new ArrayList<String>();
							tableSelectMap.put(key, list);
						}
						list.add(columnValue);
					}else {
						keyList.add(columnValue);
					}
				}
			}
			return String.join(",", keyList);
		}));
		
		Map<Integer, List<JSONObject>> allLevelIndexMap = dataList.stream().collect(Collectors.groupingBy(e -> Integer.valueOf(e.getInteger("level_index"))));
		
		JSONArray transformTendencyArr = new JSONArray();
		List<Map<String, Object>> numUserArr = new ArrayList<Map<String, Object>>();
		//拼接all-总计
		JSONObject allNumUserObject = new JSONObject();
		allNumUserObject.put("type", "全部");
		List<Integer> allIntegers = new ArrayList<>();
		allLevelIndexMap.forEach((level, levelObject) -> {
			allIntegers.add(levelObject.stream().mapToInt(i -> i.getInteger("count")).sum());
		});
		allNumUserObject.put("num", allIntegers);
		allNumUserObject.put("total", allIntegers.stream().mapToInt(Integer::intValue).sum());
		numUserArr.add(allNumUserObject);
		
		//动态查询ID->NAME
		Map<String, Object> hashMap = query(tableSelectMap);
		wdMap.forEach((key, value) -> { //此处仅拼接细分维度-总计
			String legend = formatLegend(key, hashMap);
			if(!"".equals(legend)) {
				//拼接numUser
				JSONObject numUserObject = new JSONObject();
				Map<Integer, List<JSONObject>> levelIndexMap = value.stream().collect(Collectors.groupingBy(e -> Integer.valueOf(e.getInteger("level_index"))));
				//补零
				for (int o = levelIndexMap.keySet().size(); levelIndexMap.keySet().size() < stepEventCodes.size(); o++) {
					levelIndexMap.put(o + 1, new ArrayList<>());
				}
				numUserObject.put("type", legend);
				List<Integer> integers = new ArrayList<>();
				levelIndexMap.forEach((level, levelObject) -> {
					integers.add(levelObject.stream().mapToInt(i -> i.getInteger("count")).sum());
				});
				numUserObject.put("num", integers);
				numUserObject.put("total", integers.stream().mapToInt(Integer::intValue).sum());
				numUserArr.add(numUserObject);
			}
		});
		
		wdMap.forEach((key, value) -> {
			String legend = formatLegend(key, hashMap);
//			if(!resultList.stream().anyMatch(f -> key.equals(f.get("type").toString()))) {
//				return;
//			}
			if("".equals(legend)) {
				legend = "全部";
			}
			final String textLegend = legend;  //拼接月份维度数据
			//拼接返回transformTendency
			Map<String, List<JSONObject>> dateMap = value.stream().collect(Collectors.groupingBy(e -> e.getString("dt")));
			dateMap.forEach((date, objList) -> {
				JSONObject dateObject = new JSONObject();
				// 补0
				for (int o = objList.size(); o < stepEventCodes.size(); o++) {
					JSONObject level_index = new JSONObject();
					level_index.put("dt", date);
					level_index.put("level_index", o + 1);
					level_index.put("count", 0);
					objList.add(level_index);
				}
				dateObject.put("legend", textLegend);
				
				dateObject.put("datedate", date);
				Map<Integer, Long> integerLongMap = new HashMap<>();
				objList.forEach(jsonObject -> integerLongMap.put(Integer.valueOf(jsonObject.get("level_index").toString()), Long.valueOf(jsonObject.get("count").toString())));
				JSONArray overviewArr = new JSONArray();
				integerLongMap.forEach((k, v) -> {
					JSONObject overviewObject = new JSONObject();
					if (integerLongMap.containsKey(k - 1)) {
						if (integerLongMap.get(k - 1) != 0) {
							overviewObject.put("rate", String.format("%.2f", (integerLongMap.get(k).doubleValue() / integerLongMap.get(k - 1).doubleValue()) * 100));
						} else {
							overviewObject.put("rate", "0.00");
						}
					} else if (k == 1) {
						overviewObject.put("rate", "100.00");
					} else {
						overviewObject.put("rate", "0.00");
					}
					overviewObject.put("step", k);
					overviewObject.put("users", v);
					overviewArr.add(overviewObject);
				});
				dateObject.put("overview", overviewArr);
				transformTendencyArr.add(dateObject);
			});
		
		});
		dataObject.put("numUser", numUserArr);
		dataObject.put("transformTendency", transformTendencyArr);
		return dataObject;
	}
	
	/**
	 * 分布分析
	 */
	@Override
	public JSONObject distributeAnalysis(RequestDistributeAnalysis distributeAnalysisRequest) throws ParseException {
		EventDistributeCode event = distributeAnalysisRequest.getEvent();
		if(distributeAnalysisRequest.getDistributeRange() == null 
				|| distributeAnalysisRequest.getDistributeRange().isEmpty()
				|| event.getDataType() != DistributeEventUnit.general) {
			distributeAnalysisRequest.setDistributeRange(event.getDataType().getDefaultRange());
		}
		
		//查询维度和属性
		Set<String> propIdList = new HashSet<String>();
		if(StrUtil.isNotBlank(distributeAnalysisRequest.getDimension())) {
			propIdList.add(distributeAnalysisRequest.getDimension());
		}
		if(StrUtil.isNotBlank(event.getUniqAttrId())) {
			propIdList.add(event.getUniqAttrId());
		}
		String eventCode = distributeAnalysisRequest.getEvent().getEventCode();
		Map<String, List<ParamProp>> eventPropMap = initEventProp(Arrays.asList(eventCode), 
				distributeAnalysisRequest.getFilterParameter().getConditions(), propIdList);
		
		//维度ID转code列表
		Tuple<String, Long> targetPropCodeTuple = targetPropId2Code(distributeAnalysisRequest.getDimension(), eventPropMap.get(eventCode));
		String targetPropCode = targetPropCodeTuple != null ? targetPropCodeTuple.getK() : "";
		String targetPropCodeWithComma = targetPropCodeTuple != null ? (targetPropCodeTuple.getK() + ",") : "";
		Tuple<String, Long> uniqAttrCodeTuple = targetPropId2Code(event.getUniqAttrId(), eventPropMap.get(eventCode));
		String uniqAttrCode = uniqAttrCodeTuple != null ? uniqAttrCodeTuple.getK() : "";
		
		QueryWrapper<UserEvent> queryWrapper = buildCondition(distributeAnalysisRequest.getPtype(), distributeAnalysisRequest.getFilterParameter(), eventPropMap.get(eventCode));
		queryWrapper.ge("dt", distributeAnalysisRequest.getStartDate()).le("dt", distributeAnalysisRequest.getEndDate()).eq("event", eventCode);
		
		List<String> countSql = new ArrayList<String>();
		List<EventDistributeRange> rangeList = distributeAnalysisRequest.getDistributeRange();
		for(int i = 0; i < rangeList.size(); i++) {
			EventDistributeRange range = rangeList.get(i);
			if(event.getDataType() == DistributeEventUnit.distributionTime) {
				countSql.add(String.format("sumIf(num, hours >= %s and hours < %s) as a%s", range.getStart(), range.getEnd(), i));
			}else {
				if(range.getEnd() == 0) {
					countSql.add(String.format("countIf(cnt >= %s) as a%s", range.getStart(), i));
				}else {
					countSql.add(String.format("countIf(cnt >= %s and cnt < %s) as a%s", range.getStart(), range.getEnd(), i));
				}
			}
		}
		
		List<JSONObject> resultList = null;
		if(event.getDataType() == DistributeEventUnit.distributionTime){ //时间段分布
			resultList = baseMapper.distributeAnalysisOfTime(
					targetPropCodeWithComma,
					targetPropCode, 
					String.join(",", countSql), 
					queryWrapper);
		}else if(StrUtil.isNotBlank(event.getUniqAttrId())){ //属性去重
			resultList = baseMapper.distributeAnalysisOfUniqAttr(
					uniqAttrCode,
					targetPropCodeWithComma,
					targetPropCode,
					String.join(",", countSql), 
					queryWrapper);
		}else{  //区间数据
			resultList = baseMapper.distributeAnalysis(
					targetPropCodeWithComma,
					targetPropCode, 
					String.join(",", countSql), 
					queryWrapper);
		}
		
		//保存<tableName, [columnValue]>
		Map<String, List<String>> tableSelectMap = new HashMap<String, List<String>>();
		
		List<ParamProp> paramPropList = eventPropMap.get(eventCode);
		if(paramPropList != null) {
			Optional<ParamProp> optional = paramPropList.stream().filter(f -> targetPropCodeTuple != null && f.getId().longValue() == targetPropCodeTuple.getV() && f.getDatatype() == 4).findAny();
			if(optional.isPresent()) {
				List<String> list = new ArrayList<String>();
				for (JSONObject object : resultList) {
					String columnValue = object.get(targetPropCode) == null ? "" : object.get(targetPropCode).toString();
					String legend = String.format("{%s}", columnValue);
					list.add(columnValue);
					object.put(targetPropCode, legend);
				}
				ParamProp paramProp = optional.get();
				String key = String.format("%s|%s|%s", paramProp.getDicTable(), paramProp.getJoinField(), paramProp.getTargetField());
				tableSelectMap.put(key, list);
			}
		}
		
		//动态查询ID->NAME
		Map<String, Object> hashMap = query(tableSelectMap);
		
		JSONObject result = new JSONObject();
		List<Map<String, Object>> list = new ArrayList<Map<String,Object>>();
		for (JSONObject object : resultList) {
			String legend = "全部";
			if(!"".equals(targetPropCode)) {
				if(object.get(targetPropCode) == null) {
					continue;
				}
				legend = formatLegend(object.get(targetPropCode).toString(), hashMap);
			}
			if(!"".equals(legend)) {
				JSONObject data = new JSONObject();
				data.put("legend", legend);
				List<Integer> integers = new ArrayList<>();
				for(int i = 0; i < rangeList.size(); i++) {
					integers.add(object.getInteger("a" + i));
				}
				data.put("value", integers);
				data.put("total", integers.stream().mapToInt(Integer::intValue).sum());
				list.add(data);
			}
		}
		
		result.put("list", list);
		result.put("legend", rangeList.stream().map(f -> String.format("[%s,%s)", f.getStartStr(), f.getEndStr())).collect(Collectors.toList()));
		return result;
	}
	
	/**
	 * 事件分析
	 */
	public JSONObject eventAnalysis(RequestEventAnalysis eventAnalysisRequest) {
		long startDateMillis = DateUtil.parse(eventAnalysisRequest.getStartDate(), RequestEventAnalysis.DATE_FORMAT).getTime();
		long endDateMillis = DateUtil.parse(eventAnalysisRequest.getEndDate(), RequestEventAnalysis.DATE_FORMAT).getTime();
		
		Map<String, List<ParamProp>> eventPropMap = new HashMap<String, List<ParamProp>>();
		//初始化事件和属性的关系映射表
		Map<String, EventAnalysisCode> eventCodeMap = eventAnalysisRequest.getEventCodes().stream().collect(Collectors.toMap(EventAnalysisCode::getEventCode, t -> t));
//		List<String> eventCodeList = eventAnalysisRequest.getEventCodes().stream().map(EventAnalysisCode::getEventCode).collect(Collectors.toList());
		if(eventAnalysisRequest.getFilterParameter() != null) {
			eventPropMap = initEventProp(new ArrayList<String>(eventCodeMap.keySet()), eventAnalysisRequest.getFilterParameter().getConditions(), eventAnalysisRequest.getTarget());
		}
		
		JSONObject returnResult = new JSONObject();
		JSONArray detailAll = new JSONArray();
		
		//组装查询条件
		String firstEventCode = eventAnalysisRequest.getEventCodes().get(0).getEventCode();
		List<ParamProp> propList = eventPropMap.get(firstEventCode);
		QueryWrapper<UserEvent> queryWrapper = buildCondition(eventAnalysisRequest.getPtype(), eventAnalysisRequest.getFilterParameter(), propList);
		queryWrapper.first("with d_date as dt").in("event", eventCodeMap.keySet()).ge("dt", eventAnalysisRequest.getStartDate()).le("dt", eventAnalysisRequest.getEndDate());
		
		//组装selectSql
		StringBuilder selectSqlBuilder = new StringBuilder("dt, event");
		String groupBySql = "";
		//target维度ID列表转code列表
		List<Tuple<String, Long>> targetPropCodeList = targetPropId2Code(eventAnalysisRequest.getTarget(), propList);
		if(!targetPropCodeList.isEmpty()) {
			selectSqlBuilder.append(",").append(String.join(",", targetPropCodeList.stream().map(Tuple<String, Long>::getK).collect(Collectors.toList())));
		}
		groupBySql = selectSqlBuilder.toString();
		selectSqlBuilder.append(",(case ");
		//组装聚合函数
		for(EventAnalysisCode eventAnalysisCode : eventAnalysisRequest.getEventCodes()) {
			selectSqlBuilder.append("when event = '"+eventAnalysisCode.getEventCode()+"' then " + eventAnalysisCode.getCountType().sqlSnippet + "\n");
		}
		selectSqlBuilder.append(" end) as num");
		queryWrapper.select(selectSqlBuilder.toString()).groupBy(groupBySql).orderByDesc(groupBySql);
		List<Map<String, Object>> searchResult = baseMapper.selectMaps(queryWrapper);
		
		Map<String, List<Map<String, Object>>> resultMap = searchResult.stream().collect(Collectors.groupingBy(t -> t.get("event").toString()));
		for(Entry<String, List<Map<String, Object>>> entry : resultMap.entrySet()) {
			String eventCode = entry.getKey();
			List<Map<String, Object>> dataResult = entry.getValue();
			//保存<tableName, [columnValue]>
			Map<String, List<String>> tableSelectMap = new HashMap<String, List<String>>();
			//根据查询条件聚合成
			Map<String, List<Map<String, Object>>> collect = dataResult.stream()
				.collect(Collectors.groupingBy(e -> {
					List<String> keyList = new ArrayList<String>();
					for (Tuple<String, Long> targetPropCode : targetPropCodeList) {
						if(targetPropCode.getK() != null && e.get(targetPropCode.getK()) != null) {
							String columnValue = e.get(targetPropCode.getK()).toString();
							//如果存在需要查询动态表，占位符操作，最后替换
							Optional<ParamProp> optional = propList.stream().filter(f -> f.getId().longValue() == targetPropCode.getV() && f.getDatatype() == 4).findAny();
							if(optional.isPresent()) {
								ParamProp paramProp = optional.get();
								String key = String.format("%s|%s|%s", paramProp.getDicTable(), paramProp.getJoinField(), paramProp.getTargetField());
								List<String> list = tableSelectMap.get(key);
								if(list == null) {
									list = new ArrayList<String>();
									tableSelectMap.put(key, list);
								}
								list.add(columnValue);
								keyList.add(String.format("{%s}", columnValue));
							}else {
								keyList.add(columnValue);
							}
						}
					}
					return String.join(",", keyList);
			}));
			
			//动态查询ID->NAME
			Map<String, Object> hashMap = query(tableSelectMap);
			
			for (String key : collect.keySet()) { //填充数据
				List<String> dates = collect.get(key).stream().map(m -> DateUtil.format(((LocalDate)m.get("dt")).atStartOfDay(), RequestEventAnalysis.DATE_FORMAT)).collect(Collectors.toList());
				for (long i = startDateMillis; i <= endDateMillis; i += 86400000) {
					String dataDate = DateUtil.format(new Date(i), RequestEventAnalysis.DATE_FORMAT);
					if (!dates.contains(dataDate)) {
						Map<String, Object> quitMap = new HashMap<>();
						quitMap.put("num", 0);
						quitMap.put("dt", dataDate);
						collect.get(key).add(quitMap);
					}
				}
				//对查询结果集，按照日期排序
				Collections.sort(collect.get(key), (a1, a2) -> {
					return a1.get("dt").toString().compareTo(a2.get("dt").toString());
				});
			}
			
			List<Map<String, Object>> itemArray = new ArrayList<Map<String,Object>>();
			collect.forEach((k, v) -> { //对结果集做封装处理
				String legend = "ALL";
				if(!targetPropCodeList.isEmpty()) { //维度不为空时，格式化Legend
					legend = formatLegend(k, hashMap);
				}
				if(!"".equals(legend)) { //不存在的legend，不返回
					JSONObject mergeResult = new JSONObject();
					mergeResult.put("legend", legend);
					List<Integer> countValue = new ArrayList<>();
					v.forEach(x -> countValue.add(Integer.valueOf(x.get("num").toString())));
					mergeResult.put("value", countValue);
					mergeResult.put("total", countValue.stream().reduce(Integer::sum).orElse(0));
					itemArray.add(mergeResult);
				}
			});
			//解析数据
			JSONObject eventResult = new JSONObject();
			EventAnalysisCode eventAnalysisCode = eventCodeMap.get(eventCode);
			eventResult.put(String.format("%s的%s", eventAnalysisCode.getName(), eventAnalysisCode.getCountType().message), itemArray);
			detailAll.add(eventResult);
		}
		returnResult.put("all", detailAll);   // 所有查询的结果明细
		
		// 存储起止日期
		List<String> dataList = new ArrayList<>();
		for (long i = startDateMillis; i <= endDateMillis; i += 86400000) {
			dataList.add(DateUtil.format(new Date(i), RequestEventAnalysis.DATE_FORMAT));
		}
		// 日期明细
		returnResult.put("dataList", dataList);
		return returnResult;
	}
	
	private String formatLegend(String legend, Map<String, Object> hashMap) {
		return StrUtil.format(legend, hashMap, true);
//		List<String> placeHolderList = StringKit.parser(legend);
//		for(String columnValue : placeHolderList) {
//			//先从缓存中查询
//			Object queryValue = hashMap.get(columnValue);
//			if(queryValue == null) {
//				//如果查询不到对应的值，依旧显示ID
//				queryValue = columnValue;
////				queryValue = ""; //设置为空字符串
//			}
//			legend = legend.replace("{"+columnValue+"}", queryValue.toString());
//		}
//		return legend;
	}
	
	private Map<String, Object> query(Map<String, List<String>> tableSelectMap){
		Map<String, Object> hashMap = new HashMap<String, Object>();
		for(Entry<String, List<String>> entry : tableSelectMap.entrySet()) {
			String key = entry.getKey();
			List<String> values = entry.getValue();
			
			String[] split = key.split("\\|");
			List<Map<String, Object>> queryList = dynamicQueryService.queryList(split[0], split[1], split[2], Wrappers.query().in(split[1], values));
			if(queryList != null && !queryList.isEmpty()) {
				queryList.forEach(f -> {
					hashMap.put(f.get("id").toString(), f.get("name"));
				});
			}
		}
		return hashMap;
	}
	
	/**
	 * 
	 */
	public List<String> findAidList(String ptype){
		return baseMapper.findAidList(ptype);
	}
	
	/**
	 * 初始化事件与属性的关联关系
	 * @param eventCode
	 * @param conditions
	 * @param targetList
	 * @return
	 */
	Map<String, List<ParamProp>> initEventProp(List<String> eventCodes, List<EventFilterCondition> conditions, Set<String> targetList){
		
		Map<String, List<ParamProp>> paramMap = new HashMap<String, List<ParamProp>>();
		if((conditions == null || conditions.isEmpty())  && (targetList == null || targetList.isEmpty())) {
			return paramMap;
		}
		
		List<Event> events = eventService.list(Wrappers.<Event>lambdaQuery().in(Event::getCode, eventCodes));
		
		//把查询条件和target合并处理[去重]
		Set<Long> propIdList = new HashSet<Long>();
		if(targetList != null && !targetList.isEmpty()) {
			propIdList.addAll(targetList.stream().map(Long::valueOf).collect(Collectors.toSet()));
		}
		
		if(conditions != null && !conditions.isEmpty()) {
			propIdList.addAll(conditions.stream().map(EventFilterCondition::getId).collect(Collectors.toSet()));
		}
		
		//查询公共属性
//		List<Prop> publicPropList = propService.list(Wrappers.<Prop>lambdaQuery().eq(Prop::getIsCommon, 1));
//		Map<Long, Prop> publicPropMap = publicPropList.stream().collect(Collectors.toMap(Prop::getId, Function.identity()));
		List<Prop> propList = propService.listByIds(propIdList);
		Map<Long, Prop> propMap = propList.stream().collect(Collectors.toMap(Prop::getId, Function.identity()));
		
		for(Event event : events) {
			Long eventId = event.getId();
			String eventCode = event.getCode();
			
			//查询事件属性
			List<EventPropMap> eventPropList = eventPropMapService.list(
					Wrappers.<EventPropMap>lambdaQuery().eq(EventPropMap::getEventId, eventId).in(EventPropMap::getPropId, propIdList));
			
			List<ParamProp> paramPropList = new ArrayList<ParamProp>();
			paramMap.put(eventCode, paramPropList);
			
			for(Long propId : propIdList) {
//				Prop publicProp = publicPropMap.get(propId);
				Prop prop = propMap.get(propId);
				if(prop != null && "1".equals(prop.getIsCommon())) { //公共属性
//					paramPropList.add(new ParamProp(propId, publicProp.getCode(), true));
					paramPropList.add(new ParamProp(propId, prop.getCode(), true, prop.getDatatype()));
				}else if(eventPropList.stream().anyMatch(f -> f.getPropId().longValue() == propId.longValue())){ //事件属性
					EventPropMap eventPropMap = eventPropList.stream().filter(f -> f.getPropId().longValue() == propId.longValue()).findFirst().get();
					ParamProp paramProp = new ParamProp(propId, eventPropMap.getPropPosition(), false, -1);
					if(prop != null) {
						paramProp.setDatatype(prop.getDatatype());
						paramProp.setDicTable(prop.getDicTable());
						paramProp.setJoinField(prop.getJoinField());
						paramProp.setTargetField(prop.getTargetField());
					}
					paramPropList.add(paramProp);
				} else { //其他不做处理
					
				}
			}
		}
		return paramMap;
	}

	@Override
	public List<Map<String, Object>> findTitleList(String event) {
		return baseMapper.findTitleList(event);
	}

	@Override
	public List<Map<String, Object>> listEvent(RequestWebsite requestWebsite) {
		return baseMapper.findEventList(requestWebsite);
	}
	
	
}
